# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session History Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_player_data import ModelsPlayerData
from ..models.models_proposed_proposal import ModelsProposedProposal


class ApimodelsXRayTicketMatchesResult(Model):
    """Apimodels X ray ticket matches result (apimodels.XRayTicketMatchesResult)

    Properties:
        actual_time_to_match: (ActualTimeToMatch) OPTIONAL str

        created_at: (CreatedAt) OPTIONAL str

        is_pivot: (IsPivot) OPTIONAL bool

        latencies: (Latencies) OPTIONAL Dict[str, int]

        match_pool: (MatchPool) OPTIONAL str

        matched_at: (MatchedAt) OPTIONAL str

        namespace: (Namespace) OPTIONAL str

        party_session_id: (PartySessionID) OPTIONAL str

        players: (Players) OPTIONAL List[ModelsPlayerData]

        proposed_proposal: (ProposedProposal) OPTIONAL ModelsProposedProposal

        region_preference: (RegionPreference) OPTIONAL List[str]

        ticket_attributes: (TicketAttributes) OPTIONAL Dict[str, Any]

        ticket_id: (TicketID) OPTIONAL str

        time_to_match: (TimeToMatch) OPTIONAL float
    """

    # region fields

    actual_time_to_match: str  # OPTIONAL
    created_at: str  # OPTIONAL
    is_pivot: bool  # OPTIONAL
    latencies: Dict[str, int]  # OPTIONAL
    match_pool: str  # OPTIONAL
    matched_at: str  # OPTIONAL
    namespace: str  # OPTIONAL
    party_session_id: str  # OPTIONAL
    players: List[ModelsPlayerData]  # OPTIONAL
    proposed_proposal: ModelsProposedProposal  # OPTIONAL
    region_preference: List[str]  # OPTIONAL
    ticket_attributes: Dict[str, Any]  # OPTIONAL
    ticket_id: str  # OPTIONAL
    time_to_match: float  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_actual_time_to_match(self, value: str) -> ApimodelsXRayTicketMatchesResult:
        self.actual_time_to_match = value
        return self

    def with_created_at(self, value: str) -> ApimodelsXRayTicketMatchesResult:
        self.created_at = value
        return self

    def with_is_pivot(self, value: bool) -> ApimodelsXRayTicketMatchesResult:
        self.is_pivot = value
        return self

    def with_latencies(self, value: Dict[str, int]) -> ApimodelsXRayTicketMatchesResult:
        self.latencies = value
        return self

    def with_match_pool(self, value: str) -> ApimodelsXRayTicketMatchesResult:
        self.match_pool = value
        return self

    def with_matched_at(self, value: str) -> ApimodelsXRayTicketMatchesResult:
        self.matched_at = value
        return self

    def with_namespace(self, value: str) -> ApimodelsXRayTicketMatchesResult:
        self.namespace = value
        return self

    def with_party_session_id(self, value: str) -> ApimodelsXRayTicketMatchesResult:
        self.party_session_id = value
        return self

    def with_players(
        self, value: List[ModelsPlayerData]
    ) -> ApimodelsXRayTicketMatchesResult:
        self.players = value
        return self

    def with_proposed_proposal(
        self, value: ModelsProposedProposal
    ) -> ApimodelsXRayTicketMatchesResult:
        self.proposed_proposal = value
        return self

    def with_region_preference(
        self, value: List[str]
    ) -> ApimodelsXRayTicketMatchesResult:
        self.region_preference = value
        return self

    def with_ticket_attributes(
        self, value: Dict[str, Any]
    ) -> ApimodelsXRayTicketMatchesResult:
        self.ticket_attributes = value
        return self

    def with_ticket_id(self, value: str) -> ApimodelsXRayTicketMatchesResult:
        self.ticket_id = value
        return self

    def with_time_to_match(self, value: float) -> ApimodelsXRayTicketMatchesResult:
        self.time_to_match = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "actual_time_to_match"):
            result["ActualTimeToMatch"] = str(self.actual_time_to_match)
        elif include_empty:
            result["ActualTimeToMatch"] = ""
        if hasattr(self, "created_at"):
            result["CreatedAt"] = str(self.created_at)
        elif include_empty:
            result["CreatedAt"] = ""
        if hasattr(self, "is_pivot"):
            result["IsPivot"] = bool(self.is_pivot)
        elif include_empty:
            result["IsPivot"] = False
        if hasattr(self, "latencies"):
            result["Latencies"] = {
                str(k0): int(v0) for k0, v0 in self.latencies.items()
            }
        elif include_empty:
            result["Latencies"] = {}
        if hasattr(self, "match_pool"):
            result["MatchPool"] = str(self.match_pool)
        elif include_empty:
            result["MatchPool"] = ""
        if hasattr(self, "matched_at"):
            result["MatchedAt"] = str(self.matched_at)
        elif include_empty:
            result["MatchedAt"] = ""
        if hasattr(self, "namespace"):
            result["Namespace"] = str(self.namespace)
        elif include_empty:
            result["Namespace"] = ""
        if hasattr(self, "party_session_id"):
            result["PartySessionID"] = str(self.party_session_id)
        elif include_empty:
            result["PartySessionID"] = ""
        if hasattr(self, "players"):
            result["Players"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.players
            ]
        elif include_empty:
            result["Players"] = []
        if hasattr(self, "proposed_proposal"):
            result["ProposedProposal"] = self.proposed_proposal.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["ProposedProposal"] = ModelsProposedProposal()
        if hasattr(self, "region_preference"):
            result["RegionPreference"] = [str(i0) for i0 in self.region_preference]
        elif include_empty:
            result["RegionPreference"] = []
        if hasattr(self, "ticket_attributes"):
            result["TicketAttributes"] = {
                str(k0): v0 for k0, v0 in self.ticket_attributes.items()
            }
        elif include_empty:
            result["TicketAttributes"] = {}
        if hasattr(self, "ticket_id"):
            result["TicketID"] = str(self.ticket_id)
        elif include_empty:
            result["TicketID"] = ""
        if hasattr(self, "time_to_match"):
            result["TimeToMatch"] = float(self.time_to_match)
        elif include_empty:
            result["TimeToMatch"] = 0.0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        actual_time_to_match: Optional[str] = None,
        created_at: Optional[str] = None,
        is_pivot: Optional[bool] = None,
        latencies: Optional[Dict[str, int]] = None,
        match_pool: Optional[str] = None,
        matched_at: Optional[str] = None,
        namespace: Optional[str] = None,
        party_session_id: Optional[str] = None,
        players: Optional[List[ModelsPlayerData]] = None,
        proposed_proposal: Optional[ModelsProposedProposal] = None,
        region_preference: Optional[List[str]] = None,
        ticket_attributes: Optional[Dict[str, Any]] = None,
        ticket_id: Optional[str] = None,
        time_to_match: Optional[float] = None,
        **kwargs,
    ) -> ApimodelsXRayTicketMatchesResult:
        instance = cls()
        if actual_time_to_match is not None:
            instance.actual_time_to_match = actual_time_to_match
        if created_at is not None:
            instance.created_at = created_at
        if is_pivot is not None:
            instance.is_pivot = is_pivot
        if latencies is not None:
            instance.latencies = latencies
        if match_pool is not None:
            instance.match_pool = match_pool
        if matched_at is not None:
            instance.matched_at = matched_at
        if namespace is not None:
            instance.namespace = namespace
        if party_session_id is not None:
            instance.party_session_id = party_session_id
        if players is not None:
            instance.players = players
        if proposed_proposal is not None:
            instance.proposed_proposal = proposed_proposal
        if region_preference is not None:
            instance.region_preference = region_preference
        if ticket_attributes is not None:
            instance.ticket_attributes = ticket_attributes
        if ticket_id is not None:
            instance.ticket_id = ticket_id
        if time_to_match is not None:
            instance.time_to_match = time_to_match
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsXRayTicketMatchesResult:
        instance = cls()
        if not dict_:
            return instance
        if "ActualTimeToMatch" in dict_ and dict_["ActualTimeToMatch"] is not None:
            instance.actual_time_to_match = str(dict_["ActualTimeToMatch"])
        elif include_empty:
            instance.actual_time_to_match = ""
        if "CreatedAt" in dict_ and dict_["CreatedAt"] is not None:
            instance.created_at = str(dict_["CreatedAt"])
        elif include_empty:
            instance.created_at = ""
        if "IsPivot" in dict_ and dict_["IsPivot"] is not None:
            instance.is_pivot = bool(dict_["IsPivot"])
        elif include_empty:
            instance.is_pivot = False
        if "Latencies" in dict_ and dict_["Latencies"] is not None:
            instance.latencies = {
                str(k0): int(v0) for k0, v0 in dict_["Latencies"].items()
            }
        elif include_empty:
            instance.latencies = {}
        if "MatchPool" in dict_ and dict_["MatchPool"] is not None:
            instance.match_pool = str(dict_["MatchPool"])
        elif include_empty:
            instance.match_pool = ""
        if "MatchedAt" in dict_ and dict_["MatchedAt"] is not None:
            instance.matched_at = str(dict_["MatchedAt"])
        elif include_empty:
            instance.matched_at = ""
        if "Namespace" in dict_ and dict_["Namespace"] is not None:
            instance.namespace = str(dict_["Namespace"])
        elif include_empty:
            instance.namespace = ""
        if "PartySessionID" in dict_ and dict_["PartySessionID"] is not None:
            instance.party_session_id = str(dict_["PartySessionID"])
        elif include_empty:
            instance.party_session_id = ""
        if "Players" in dict_ and dict_["Players"] is not None:
            instance.players = [
                ModelsPlayerData.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["Players"]
            ]
        elif include_empty:
            instance.players = []
        if "ProposedProposal" in dict_ and dict_["ProposedProposal"] is not None:
            instance.proposed_proposal = ModelsProposedProposal.create_from_dict(
                dict_["ProposedProposal"], include_empty=include_empty
            )
        elif include_empty:
            instance.proposed_proposal = ModelsProposedProposal()
        if "RegionPreference" in dict_ and dict_["RegionPreference"] is not None:
            instance.region_preference = [str(i0) for i0 in dict_["RegionPreference"]]
        elif include_empty:
            instance.region_preference = []
        if "TicketAttributes" in dict_ and dict_["TicketAttributes"] is not None:
            instance.ticket_attributes = {
                str(k0): v0 for k0, v0 in dict_["TicketAttributes"].items()
            }
        elif include_empty:
            instance.ticket_attributes = {}
        if "TicketID" in dict_ and dict_["TicketID"] is not None:
            instance.ticket_id = str(dict_["TicketID"])
        elif include_empty:
            instance.ticket_id = ""
        if "TimeToMatch" in dict_ and dict_["TimeToMatch"] is not None:
            instance.time_to_match = float(dict_["TimeToMatch"])
        elif include_empty:
            instance.time_to_match = 0.0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsXRayTicketMatchesResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsXRayTicketMatchesResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsXRayTicketMatchesResult,
        List[ApimodelsXRayTicketMatchesResult],
        Dict[Any, ApimodelsXRayTicketMatchesResult],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "ActualTimeToMatch": "actual_time_to_match",
            "CreatedAt": "created_at",
            "IsPivot": "is_pivot",
            "Latencies": "latencies",
            "MatchPool": "match_pool",
            "MatchedAt": "matched_at",
            "Namespace": "namespace",
            "PartySessionID": "party_session_id",
            "Players": "players",
            "ProposedProposal": "proposed_proposal",
            "RegionPreference": "region_preference",
            "TicketAttributes": "ticket_attributes",
            "TicketID": "ticket_id",
            "TimeToMatch": "time_to_match",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "ActualTimeToMatch": False,
            "CreatedAt": False,
            "IsPivot": False,
            "Latencies": False,
            "MatchPool": False,
            "MatchedAt": False,
            "Namespace": False,
            "PartySessionID": False,
            "Players": False,
            "ProposedProposal": False,
            "RegionPreference": False,
            "TicketAttributes": False,
            "TicketID": False,
            "TimeToMatch": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsBatchDownloadLogsRequest
from ..models import ModelsListTerminatedServersResponse
from ..models import ModelsMetadataServersRequest
from ..models import ResponseError

from ..operations.all_terminated_servers import BatchDownloadServerLogs
from ..operations.all_terminated_servers import ListAllTerminatedServers
from ..operations.all_terminated_servers import ListMetadataServers


@same_doc_as(BatchDownloadServerLogs)
def batch_download_server_logs(
    body: ModelsBatchDownloadLogsRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Batch Download dedicated server log files (batchDownloadServerLogs)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSLM:LOG [READ]

    Required scope: social

    This endpoint will download dedicated server's log file (.zip).

    Properties:
        url: /dslogmanager/servers/logs/download

        method: POST

        tags: ["All Terminated Servers"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBatchDownloadLogsRequest in body

    Responses:
        200: OK - Any (server logs downloaded.)

        400: Bad Request - ResponseError (Bad Request)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = BatchDownloadServerLogs.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BatchDownloadServerLogs)
async def batch_download_server_logs_async(
    body: ModelsBatchDownloadLogsRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Batch Download dedicated server log files (batchDownloadServerLogs)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSLM:LOG [READ]

    Required scope: social

    This endpoint will download dedicated server's log file (.zip).

    Properties:
        url: /dslogmanager/servers/logs/download

        method: POST

        tags: ["All Terminated Servers"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBatchDownloadLogsRequest in body

    Responses:
        200: OK - Any (server logs downloaded.)

        400: Bad Request - ResponseError (Bad Request)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = BatchDownloadServerLogs.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListAllTerminatedServers)
def list_all_terminated_servers(
    deployment: Optional[str] = None,
    end_date: Optional[str] = None,
    game_mode: Optional[str] = None,
    limit: Optional[int] = None,
    namespace: Optional[str] = None,
    next_: Optional[str] = None,
    party_id: Optional[str] = None,
    pod_name: Optional[str] = None,
    previous: Optional[str] = None,
    provider: Optional[str] = None,
    region: Optional[str] = None,
    session_id: Optional[str] = None,
    start_date: Optional[str] = None,
    status: Optional[str] = None,
    user_id: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Terminated Servers (listAllTerminatedServers)

    ```
    Required permission: ADMIN:NAMESPACE:{namespace}:DSLM:SERVER [READ]

    This endpoint used to retrieve terminated servers in all namespace
    ```

    Properties:
        url: /dslogmanager/servers/search

        method: GET

        tags: ["All Terminated Servers"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) OPTIONAL str in query

        end_date: (end_date) OPTIONAL str in query

        game_mode: (game_mode) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        namespace: (namespace) OPTIONAL str in query

        next_: (next) OPTIONAL str in query

        party_id: (party_id) OPTIONAL str in query

        pod_name: (pod_name) OPTIONAL str in query

        previous: (previous) OPTIONAL str in query

        provider: (provider) OPTIONAL str in query

        region: (region) OPTIONAL str in query

        session_id: (session_id) OPTIONAL str in query

        start_date: (start_date) OPTIONAL str in query

        status: (status) OPTIONAL str in query

        user_id: (user_id) OPTIONAL str in query

    Responses:
        200: OK - ModelsListTerminatedServersResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = ListAllTerminatedServers.create(
        deployment=deployment,
        end_date=end_date,
        game_mode=game_mode,
        limit=limit,
        namespace=namespace,
        next_=next_,
        party_id=party_id,
        pod_name=pod_name,
        previous=previous,
        provider=provider,
        region=region,
        session_id=session_id,
        start_date=start_date,
        status=status,
        user_id=user_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListAllTerminatedServers)
async def list_all_terminated_servers_async(
    deployment: Optional[str] = None,
    end_date: Optional[str] = None,
    game_mode: Optional[str] = None,
    limit: Optional[int] = None,
    namespace: Optional[str] = None,
    next_: Optional[str] = None,
    party_id: Optional[str] = None,
    pod_name: Optional[str] = None,
    previous: Optional[str] = None,
    provider: Optional[str] = None,
    region: Optional[str] = None,
    session_id: Optional[str] = None,
    start_date: Optional[str] = None,
    status: Optional[str] = None,
    user_id: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Terminated Servers (listAllTerminatedServers)

    ```
    Required permission: ADMIN:NAMESPACE:{namespace}:DSLM:SERVER [READ]

    This endpoint used to retrieve terminated servers in all namespace
    ```

    Properties:
        url: /dslogmanager/servers/search

        method: GET

        tags: ["All Terminated Servers"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment: (deployment) OPTIONAL str in query

        end_date: (end_date) OPTIONAL str in query

        game_mode: (game_mode) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        namespace: (namespace) OPTIONAL str in query

        next_: (next) OPTIONAL str in query

        party_id: (party_id) OPTIONAL str in query

        pod_name: (pod_name) OPTIONAL str in query

        previous: (previous) OPTIONAL str in query

        provider: (provider) OPTIONAL str in query

        region: (region) OPTIONAL str in query

        session_id: (session_id) OPTIONAL str in query

        start_date: (start_date) OPTIONAL str in query

        status: (status) OPTIONAL str in query

        user_id: (user_id) OPTIONAL str in query

    Responses:
        200: OK - ModelsListTerminatedServersResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = ListAllTerminatedServers.create(
        deployment=deployment,
        end_date=end_date,
        game_mode=game_mode,
        limit=limit,
        namespace=namespace,
        next_=next_,
        party_id=party_id,
        pod_name=pod_name,
        previous=previous,
        provider=provider,
        region=region,
        session_id=session_id,
        start_date=start_date,
        status=status,
        user_id=user_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListMetadataServers)
def list_metadata_servers(
    body: ModelsMetadataServersRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Metadata Servers (listMetadataServers)

    ```
    Required permission: ADMIN:NAMESPACE:{namespace}:DSLM:SERVER [READ]

    This endpoint used to retrieve metadata servers in a namespace

    The namespace filter is will give result exact namespace response
    ```

    Properties:
        url: /dslogmanager/servers/metadata

        method: POST

        tags: ["All Terminated Servers"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsMetadataServersRequest in body

    Responses:
        200: OK - ModelsListTerminatedServersResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = ListMetadataServers.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListMetadataServers)
async def list_metadata_servers_async(
    body: ModelsMetadataServersRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Metadata Servers (listMetadataServers)

    ```
    Required permission: ADMIN:NAMESPACE:{namespace}:DSLM:SERVER [READ]

    This endpoint used to retrieve metadata servers in a namespace

    The namespace filter is will give result exact namespace response
    ```

    Properties:
        url: /dslogmanager/servers/metadata

        method: POST

        tags: ["All Terminated Servers"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsMetadataServersRequest in body

    Responses:
        200: OK - ModelsListTerminatedServersResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = ListMetadataServers.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2024 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
import asyncio
from typing import Optional

from ...core import WSClient, WSClientProtocol, WebSocketMessageParserException
from ...core import TokenRepositoryObserver

from .wss_models import ConnectNotif, RefreshTokenRequest


class LobbyWSClient(WSClient):
    LOBBY_SESSION_ID_DATA_KEY: str = "lobby_session_id"
    LOBBY_SESSION_ID_HEADER_KEY: str = "X-Ab-LobbySessionID"

    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

    async def connect(self, *, reconnecting: bool = False, **kwargs) -> bool:
        url_suffix = kwargs.pop("url_suffix", "/lobby/")
        extra_headers = kwargs.pop("extra_headers", {})

        if reconnecting and self.has_data(self.LOBBY_SESSION_ID_DATA_KEY):
            if lobby_session_id := self.get_data(self.LOBBY_SESSION_ID_DATA_KEY):
                extra_headers[self.LOBBY_SESSION_ID_HEADER_KEY] = lobby_session_id

        if not extra_headers:
            extra_headers = None

        return await super().connect(
            reconnecting=reconnecting,
            url_suffix=url_suffix,
            extra_headers=extra_headers,
            **kwargs,
        )

    async def on_message(self, message: str, **kwargs) -> None:
        if message.startswith("type: connectNotif"):
            try:
                connect_notif = ConnectNotif.create_from_wsm(message, is_strict=False)
                if connect_notif.lobby_session_id:
                    self.set_data(
                        self.LOBBY_SESSION_ID_DATA_KEY, connect_notif.lobby_session_id
                    )
            except WebSocketMessageParserException as error:
                pass  # explicitly ignore

        await super().on_message(message=message, **kwargs)

    def on_access_token_changed(self, access_token: Optional[str]) -> None:
        if not access_token:
            return

        refresh_token_request = RefreshTokenRequest()
        refresh_token_request.token = access_token

        asyncio.ensure_future(self.send(message=refresh_token_request.to_wsm()))


assert isinstance(LobbyWSClient, WSClientProtocol)
assert isinstance(LobbyWSClient, TokenRepositoryObserver)


__all__ = [
    "LobbyWSClient",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Basic Service."""

__version__ = "4.0.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .change_namespace_status import ChangeNamespaceStatus
from .create_namespace import CreateNamespace
from .delete_namespace import DeleteNamespace
from .get_child_namespaces import GetChildNamespaces
from .get_game_namespaces import GetGameNamespaces
from .get_namespace import GetNamespace
from .get_namespace_1 import GetNamespace1
from .get_namespace_context import GetNamespaceContext
from .get_namespace_publisher import GetNamespacePublisher
from .get_namespaces import GetNamespaces
from .public_get_namespace_publisher import PublicGetNamespacePublisher
from .public_get_namespaces import PublicGetNamespaces
from .update_namespace import UpdateNamespace



from threading import RLock
from typing import Any, Dict, List, Optional, Union

import jwt

import accelbyte_py_sdk.api.basic as basic_service
import accelbyte_py_sdk.api.basic.models as basic_models
import accelbyte_py_sdk.api.iam as iam_service
import accelbyte_py_sdk.api.iam.models as iam_models
from accelbyte_py_sdk.core import AccelByteSDK, Timer

from ._bloom_filter import BloomFilter
from ._ctypes import (
    FetchNamespaceContextError,
    FetchRoleError,
    PermissionStruct,
    create_permission_struct,
)
from ._utils import replace_resource, str2datetime

PublicPrivateKey = Any
JWTClaims = Dict[str, Any]
JWKSet = Dict[str, PublicPrivateKey]
NamespaceContext = basic_models.NamespaceContext
NamespaceRole = Dict[str, str]
Role = iam_models.ModelRoleResponseV3


class JWKSCache(Timer):
    JWKS_KEYS_KEY: str = "keys"

    def __init__(
        self,
        sdk: AccelByteSDK,
        interval: Optional[Union[int, float]] = None,
        raise_on_error: bool = False,
    ):
        self.sdk = sdk
        self._jwks: Dict[str, PublicPrivateKey] = {}
        self._lock = RLock()
        self._raise_on_error = raise_on_error
        if interval is not None:
            Timer.__init__(
                self,
                interval,
                self.update,
                daemon=True,
                repeats=-1,
                autostart=True,
                repeat_on_exception=True,
            )

    def update(self):
        result, error = iam_service.get_jwksv3(sdk=self.sdk)
        if error:
            if self._raise_on_error:
                raise Exception(error)
            return

        with self._lock:
            keys = result.to_dict().get(JWKSCache.JWKS_KEYS_KEY, [])
            jwks = jwt.PyJWKSet(keys)
            for jwk in jwks.keys:
                self._jwks[jwk.key_id] = jwk.key

    def get_key_from_cache(self, key_id: str) -> Optional[PublicPrivateKey]:
        with self._lock:
            return self._jwks.get(key_id, None)

    def get_key(self, key_id: str) -> Optional[PublicPrivateKey]:
        key = self.get_key_from_cache(key_id)
        if key is not None:
            return key

        self.update()

        return self.get_key_from_cache(key_id)


class NamespaceContextCache(Timer):
    def __init__(
        self,
        sdk: AccelByteSDK,
        interval: Union[int, float],
        raise_on_error: bool = True,
    ):
        self.sdk = sdk
        self._namespace_contexts: Dict[str, Any] = {}
        self._lock = RLock()
        self._raise_on_error = raise_on_error
        Timer.__init__(
            self,
            interval,
            self.clear,
            daemon=True,
            repeats=-1,
            autostart=True,
            repeat_on_exception=True,
        )

    def clear(self):
        with self._lock:
            self._namespace_contexts = {}

    def update(self):
        pass

    def cache_namespace_context(self, namespace: str) -> Any:
        namespace_context, error = basic_service.get_namespace_context(
            namespace=namespace, sdk=self.sdk
        )
        if error:
            return error
        with self._lock:
            self._namespace_contexts[namespace] = namespace_context
        return None

    def get_namespace_context(self, namespace: str) -> Optional[NamespaceContext]:
        with self._lock:
            namespace_context = self._namespace_contexts.get(namespace, None)
            if namespace_context:
                return namespace_context

        error = self.cache_namespace_context(namespace=namespace)
        if error:
            if self._raise_on_error:
                raise FetchNamespaceContextError(f"namespace context: {namespace}")
            return None

        return self._namespace_contexts.get(namespace)


class RevocationListCache(Timer):
    def __init__(
        self,
        sdk: AccelByteSDK,
        interval: Union[int, float],
        raise_on_error: bool = True,
    ):
        self.sdk = sdk
        self._revoked_token_filter: Optional[BloomFilter] = None
        self._revoked_users: Dict[str, float] = {}
        self._lock = RLock()
        self._raise_on_error = raise_on_error
        Timer.__init__(
            self,
            interval,
            self.update,
            daemon=True,
            repeats=-1,
            autostart=True,
            repeat_on_exception=True,
        )

    def update(self):
        result, error = iam_service.get_revocation_list_v3(sdk=self.sdk)
        if error:
            return

        with self._lock:
            # Revoked Tokens
            revoked_tokens = result.revoked_tokens
            self._revoked_token_filter = BloomFilter.create_from_bits(
                bits=revoked_tokens.bits, k=revoked_tokens.k, m=revoked_tokens.m
            )
            # Revoked Users
            revoked_users = result.revoked_users
            self._revoked_users = {}
            for user in revoked_users:
                if user.id_ and user.revoked_at:
                    revoked_at = str2datetime(user.revoked_at).timestamp()
                    self._revoked_users[user.id_] = revoked_at

    def is_token_revoked(self, token: str) -> bool:
        with self._lock:
            return self._revoked_token_filter.might_contains(key=token)

    def is_user_revoked(self, user_id: str, issued_at: int) -> bool:
        with self._lock:
            revoked_at = self._revoked_users.get(user_id)
            if revoked_at:
                return revoked_at >= issued_at
        return False


class RolesCache(Timer):
    def __init__(
        self,
        sdk: AccelByteSDK,
        interval: Union[int, float],
        raise_on_error: bool = True,
    ):
        self.sdk = sdk
        self._roles: Dict[str, Any] = {}
        self._lock = RLock()
        self._raise_on_error = raise_on_error
        Timer.__init__(
            self,
            interval,
            self.clear,
            daemon=True,
            repeats=-1,
            autostart=True,
            repeat_on_exception=True,
        )

    def clear(self):
        with self._lock:
            self._roles = {}

    def update(self):
        pass

    def cache_role(self, role_id: str) -> Any:
        role, error = iam_service.admin_get_role_v3(role_id=role_id, sdk=self.sdk)
        if error:
            return error
        with self._lock:
            self._roles[role_id] = role
        return None

    def get_role(self, role_id: str) -> Optional[Role]:
        with self._lock:
            role = self._roles.get(role_id, None)
            if role:
                return role

        error = self.cache_role(role_id=role_id)
        if error:
            if self._raise_on_error:
                raise FetchRoleError(f"role ID: {role_id}")
            return None

        return self._roles.get(role_id)

    def get_role_permissions(self, role_id: str) -> List[PermissionStruct]:
        role = self.get_role(role_id=role_id)
        return role.permissions

    def get_modified_role_permissions(
        self, role_id: str, namespace: str, user_id: Optional[str] = None
    ) -> List[PermissionStruct]:
        permissions = self.get_role_permissions(role_id=role_id)
        return [
            create_permission_struct(
                action=p.action,
                resource=replace_resource(
                    resource=p.resource,
                    namespace=namespace,
                    user_id=user_id,
                ),
            )
            for p in permissions
        ]

    def get_modified_role_permissions2(
        self, namespace_role: NamespaceRole, user_id: Optional[str] = None
    ) -> List[PermissionStruct]:
        return self.get_modified_role_permissions(
            role_id=namespace_role.get("roleId"),
            namespace=namespace_role.get("namespace"),
            user_id=user_id,
        )


__all__ = [
    # Type Aliases
    "JWTClaims",
    "JWKSet",
    "NamespaceContext",
    "NamespaceRole",
    "PublicPrivateKey",
    "Role",
    # Cache Types
    "JWKSCache",
    "NamespaceContextCache",
    "RevocationListCache",
    "RolesCache",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Gdpr Service."""

__version__ = "2.15.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_cancel_user_perso_78952d import AdminCancelUserPersonalDataRequest
from .admin_generate_personal_48c32b import AdminGeneratePersonalDataURL
from .admin_get_list_personal_424fda import AdminGetListPersonalDataRequest
from .admin_get_user_personal_a892c0 import AdminGetUserPersonalDataRequests
from .admin_request_data_retrieval import AdminRequestDataRetrieval
from .public_cancel_user_pers_19dafa import PublicCancelUserPersonalDataRequest
from .public_generate_persona_6b68a4 import PublicGeneratePersonalDataURL
from .public_get_user_persona_7e40c3 import PublicGetUserPersonalDataRequests
from .public_request_data_retrieval import PublicRequestDataRetrieval



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import ModelDeviceBanRequestV4
from ..models import ModelDeviceBanResponseV4
from ..models import ModelDeviceBanUpdateRequestV4
from ..models import ModelDeviceBannedResponseV4
from ..models import ModelDeviceBansResponseV4
from ..models import ModelDeviceIDDecryptResponseV4
from ..models import ModelDeviceTypesResponseV4
from ..models import ModelDeviceUsersResponseV4
from ..models import ModelDevicesResponseV4
from ..models import RestErrorResponse

from ..operations.devices_v4 import AdminBanDeviceV4
from ..operations.devices_v4 import AdminDecryptDeviceV4
from ..operations.devices_v4 import AdminGenerateReportV4
from ..operations.devices_v4 import AdminGetBannedDevicesV4
from ..operations.devices_v4 import AdminGetDeviceBanV4
from ..operations.devices_v4 import AdminGetDeviceBansV4
from ..operations.devices_v4 import AdminGetDeviceTypesV4
from ..operations.devices_v4 import AdminGetDevicesByUserV4
from ..operations.devices_v4 import AdminGetUserDeviceBansV4
from ..operations.devices_v4 import AdminGetUsersByDeviceV4
from ..operations.devices_v4 import AdminUnbanDeviceV4
from ..operations.devices_v4 import AdminUpdateDeviceBanV4


@same_doc_as(AdminBanDeviceV4)
def admin_ban_device_v4(
    body: ModelDeviceBanRequestV4,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin Ban a device (AdminBanDeviceV4)

    This is the endpoint for an admin to ban a device

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/bans

        method: POST

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelDeviceBanRequestV4 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error | 10204: device can not be banned)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        409: Conflict - RestErrorResponse (10202: active device ban config already exists)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminBanDeviceV4.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminBanDeviceV4)
async def admin_ban_device_v4_async(
    body: ModelDeviceBanRequestV4,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin Ban a device (AdminBanDeviceV4)

    This is the endpoint for an admin to ban a device

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/bans

        method: POST

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelDeviceBanRequestV4 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error | 10204: device can not be banned)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        409: Conflict - RestErrorResponse (10202: active device ban config already exists)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminBanDeviceV4.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(AdminDecryptDeviceV4)
def admin_decrypt_device_v4(
    device_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin decrypt device id (AdminDecryptDeviceV4)

    This is the endpoint for an admin to decrypt device id

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/{deviceId}/decrypt

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        device_id: (deviceId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceIDDecryptResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminDecryptDeviceV4.create(
        device_id=device_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(AdminDecryptDeviceV4)
async def admin_decrypt_device_v4_async(
    device_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin decrypt device id (AdminDecryptDeviceV4)

    This is the endpoint for an admin to decrypt device id

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/{deviceId}/decrypt

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        device_id: (deviceId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceIDDecryptResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminDecryptDeviceV4.create(
        device_id=device_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGenerateReportV4)
def admin_generate_report_v4(
    device_type: str,
    end_date: Optional[str] = None,
    start_date: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin generate device report (AdminGenerateReportV4)

    This is the endpoint for an admin to generate device report

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/report

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_date: (endDate) OPTIONAL str in query

        start_date: (startDate) OPTIONAL str in query

        device_type: (deviceType) REQUIRED str in query

    Responses:
        200: OK - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (20008: user not found | 10139: platform account not found | 10158: ban not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGenerateReportV4.create(
        device_type=device_type,
        end_date=end_date,
        start_date=start_date,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGenerateReportV4)
async def admin_generate_report_v4_async(
    device_type: str,
    end_date: Optional[str] = None,
    start_date: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin generate device report (AdminGenerateReportV4)

    This is the endpoint for an admin to generate device report

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/report

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_date: (endDate) OPTIONAL str in query

        start_date: (startDate) OPTIONAL str in query

        device_type: (deviceType) REQUIRED str in query

    Responses:
        200: OK - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (20008: user not found | 10139: platform account not found | 10158: ban not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGenerateReportV4.create(
        device_type=device_type,
        end_date=end_date,
        start_date=start_date,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetBannedDevicesV4)
def admin_get_banned_devices_v4(
    device_type: Optional[str] = None,
    end_date: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    start_date: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get banned devices (AdminGetBannedDevicesV4)

    This is the endpoint for an admin to get banned devices

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/banned

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        device_type: (deviceType) OPTIONAL str in query

        end_date: (endDate) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        start_date: (startDate) OPTIONAL str in query

    Responses:
        200: OK - ModelDeviceBannedResponseV4 (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetBannedDevicesV4.create(
        device_type=device_type,
        end_date=end_date,
        limit=limit,
        offset=offset,
        start_date=start_date,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetBannedDevicesV4)
async def admin_get_banned_devices_v4_async(
    device_type: Optional[str] = None,
    end_date: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    start_date: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get banned devices (AdminGetBannedDevicesV4)

    This is the endpoint for an admin to get banned devices

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/banned

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        device_type: (deviceType) OPTIONAL str in query

        end_date: (endDate) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        start_date: (startDate) OPTIONAL str in query

    Responses:
        200: OK - ModelDeviceBannedResponseV4 (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetBannedDevicesV4.create(
        device_type=device_type,
        end_date=end_date,
        limit=limit,
        offset=offset,
        start_date=start_date,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetDeviceBanV4)
def admin_get_device_ban_v4(
    ban_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get device ban config (AdminGetDeviceBanV4)

    This is the endpoint for an admin to get device ban config

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/bans/{banId}

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        ban_id: (banId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceBanResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10158: ban not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetDeviceBanV4.create(
        ban_id=ban_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetDeviceBanV4)
async def admin_get_device_ban_v4_async(
    ban_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get device ban config (AdminGetDeviceBanV4)

    This is the endpoint for an admin to get device ban config

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/bans/{banId}

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        ban_id: (banId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceBanResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10158: ban not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetDeviceBanV4.create(
        ban_id=ban_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetDeviceBansV4)
def admin_get_device_bans_v4(
    device_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get device ban list (AdminGetDeviceBansV4)

    This is the endpoint for an admin to get device ban list

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/{deviceId}/bans

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        device_id: (deviceId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceBansResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetDeviceBansV4.create(
        device_id=device_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetDeviceBansV4)
async def admin_get_device_bans_v4_async(
    device_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get device ban list (AdminGetDeviceBansV4)

    This is the endpoint for an admin to get device ban list

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/{deviceId}/bans

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        device_id: (deviceId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceBansResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetDeviceBansV4.create(
        device_id=device_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetDeviceTypesV4)
def admin_get_device_types_v4(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get device types (AdminGetDeviceTypesV4)

    This is the endpoint for an admin to get device types

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/types

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceTypesResponseV4 (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetDeviceTypesV4.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetDeviceTypesV4)
async def admin_get_device_types_v4_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get device types (AdminGetDeviceTypesV4)

    This is the endpoint for an admin to get device types

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/types

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceTypesResponseV4 (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetDeviceTypesV4.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetDevicesByUserV4)
def admin_get_devices_by_user_v4(
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get devices by user id (AdminGetDevicesByUserV4)

    This is the endpoint for an admin to get devices a user ever used to login

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - ModelDevicesResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error | 20021: invalid pagination parameter)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10139: platform account not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetDevicesByUserV4.create(
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetDevicesByUserV4)
async def admin_get_devices_by_user_v4_async(
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get devices by user id (AdminGetDevicesByUserV4)

    This is the endpoint for an admin to get devices a user ever used to login

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - ModelDevicesResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error | 20021: invalid pagination parameter)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10139: platform account not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetDevicesByUserV4.create(
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetUserDeviceBansV4)
def admin_get_user_device_bans_v4(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get device bans of user (AdminGetUserDeviceBansV4)

    This is the endpoint for an admin to get device bans of user

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/bans

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in query

    Responses:
        200: OK - ModelDeviceBansResponseV4 (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10139: platform account not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetUserDeviceBansV4.create(
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetUserDeviceBansV4)
async def admin_get_user_device_bans_v4_async(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get device bans of user (AdminGetUserDeviceBansV4)

    This is the endpoint for an admin to get device bans of user

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/bans

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in query

    Responses:
        200: OK - ModelDeviceBansResponseV4 (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10139: platform account not found)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetUserDeviceBansV4.create(
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetUsersByDeviceV4)
def admin_get_users_by_device_v4(
    device_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get users by device id (AdminGetUsersByDeviceV4)

    This is the endpoint for an admin to get users that ever login on the device

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/{deviceId}/users

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        device_id: (deviceId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceUsersResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetUsersByDeviceV4.create(
        device_id=device_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetUsersByDeviceV4)
async def admin_get_users_by_device_v4_async(
    device_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin get users by device id (AdminGetUsersByDeviceV4)

    This is the endpoint for an admin to get users that ever login on the device

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/{deviceId}/users

        method: GET

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        device_id: (deviceId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelDeviceUsersResponseV4 (Operation succeeded)

        400: Bad Request - RestErrorResponse (20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetUsersByDeviceV4.create(
        device_id=device_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUnbanDeviceV4)
def admin_unban_device_v4(
    device_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin unban device (AdminUnbanDeviceV4)

    This is the endpoint for an admin to unban device

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/{deviceId}/unban

        method: PUT

        tags: ["Devices V4"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        device_id: (deviceId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUnbanDeviceV4.create(
        device_id=device_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUnbanDeviceV4)
async def admin_unban_device_v4_async(
    device_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin unban device (AdminUnbanDeviceV4)

    This is the endpoint for an admin to unban device

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/{deviceId}/unban

        method: PUT

        tags: ["Devices V4"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        device_id: (deviceId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUnbanDeviceV4.create(
        device_id=device_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateDeviceBanV4)
def admin_update_device_ban_v4(
    ban_id: str,
    body: ModelDeviceBanUpdateRequestV4,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin update device ban config (AdminUpdateDeviceBanV4)

    This is the endpoint for an admin to update a device ban config

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/bans/{banId}

        method: PUT

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelDeviceBanUpdateRequestV4 in body

        ban_id: (banId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10158: ban not found)

        409: Conflict - RestErrorResponse (10202: active device ban config already exists)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateDeviceBanV4.create(
        ban_id=ban_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateDeviceBanV4)
async def admin_update_device_ban_v4_async(
    ban_id: str,
    body: ModelDeviceBanUpdateRequestV4,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin update device ban config (AdminUpdateDeviceBanV4)

    This is the endpoint for an admin to update a device ban config

    Properties:
        url: /iam/v4/admin/namespaces/{namespace}/devices/bans/{banId}

        method: PUT

        tags: ["Devices V4"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelDeviceBanUpdateRequestV4 in body

        ban_id: (banId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Operation succeeded)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)

        404: Not Found - RestErrorResponse (10158: ban not found)

        409: Conflict - RestErrorResponse (10202: active device ban config already exists)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateDeviceBanV4.create(
        ban_id=ban_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelsCreateInventoryReq
from ..models import ApimodelsDeleteInventoryReq
from ..models import ApimodelsErrorResponse
from ..models import ApimodelsInventoryResp
from ..models import ApimodelsListInventoryResp
from ..models import ApimodelsPurchaseValidationReq
from ..models import ApimodelsUpdateInventoryReq

from ..operations.admin_inventories import AdminCreateInventory
from ..operations.admin_inventories import AdminGetInventory
from ..operations.admin_inventories import AdminListInventories
from ..operations.admin_inventories import AdminListInventoriesSortByEnum
from ..operations.admin_inventories import AdminPurchasable
from ..operations.admin_inventories import AdminUpdateInventory
from ..operations.admin_inventories import AdminUpdateUserInventoriesByInventoryCode
from ..operations.admin_inventories import DeleteInventory


@same_doc_as(AdminCreateInventory)
def admin_create_inventory(
    body: ApimodelsCreateInventoryReq,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To create inventory (AdminCreateInventory)


    Creating an inventory.
    The inventory configuration must exists otherwise it will fail.
    The max slots and max upgrade slots of an inventory will be initialized according to the inventory configuration it used,
    but it can be changed later when using AdminUpdateInventory endpoint.

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [CREATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories

        method: POST

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsCreateInventoryReq in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApimodelsInventoryResp (Created)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminCreateInventory.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateInventory)
async def admin_create_inventory_async(
    body: ApimodelsCreateInventoryReq,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To create inventory (AdminCreateInventory)


    Creating an inventory.
    The inventory configuration must exists otherwise it will fail.
    The max slots and max upgrade slots of an inventory will be initialized according to the inventory configuration it used,
    but it can be changed later when using AdminUpdateInventory endpoint.

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [CREATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories

        method: POST

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsCreateInventoryReq in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApimodelsInventoryResp (Created)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminCreateInventory.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetInventory)
def admin_get_inventory(
    inventory_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To get an inventory (AdminGetInventory)


    Getting an inventory info.

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [READ]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories/{inventoryId}

        method: GET

        tags: ["Admin Inventories"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        inventory_id: (inventoryId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsInventoryResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetInventory.create(
        inventory_id=inventory_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetInventory)
async def admin_get_inventory_async(
    inventory_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To get an inventory (AdminGetInventory)


    Getting an inventory info.

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [READ]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories/{inventoryId}

        method: GET

        tags: ["Admin Inventories"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        inventory_id: (inventoryId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsInventoryResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetInventory.create(
        inventory_id=inventory_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminListInventories)
def admin_list_inventories(
    inventory_configuration_code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, AdminListInventoriesSortByEnum]] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To list all inventories (AdminListInventories)


    Listing all inventories in a namespace.
    The response body will be in the form of standard pagination.

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [READ]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories

        method: GET

        tags: ["Admin Inventories"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        inventory_configuration_code: (inventoryConfigurationCode) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - ApimodelsListInventoryResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminListInventories.create(
        inventory_configuration_code=inventory_configuration_code,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminListInventories)
async def admin_list_inventories_async(
    inventory_configuration_code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, AdminListInventoriesSortByEnum]] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To list all inventories (AdminListInventories)


    Listing all inventories in a namespace.
    The response body will be in the form of standard pagination.

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [READ]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories

        method: GET

        tags: ["Admin Inventories"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        inventory_configuration_code: (inventoryConfigurationCode) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - ApimodelsListInventoryResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminListInventories.create(
        inventory_configuration_code=inventory_configuration_code,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPurchasable)
def admin_purchasable(
    body: ApimodelsPurchaseValidationReq,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To validate user inventory capacity when purchase ecommerce item (AdminPurchasable)


    Validate purchase ecommerce item.

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [UPDATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/users/{userId}/purchaseable

        method: POST

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsPurchaseValidationReq in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        409: Conflict - ApimodelsErrorResponse (Conflict)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPurchasable.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPurchasable)
async def admin_purchasable_async(
    body: ApimodelsPurchaseValidationReq,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To validate user inventory capacity when purchase ecommerce item (AdminPurchasable)


    Validate purchase ecommerce item.

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [UPDATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/users/{userId}/purchaseable

        method: POST

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsPurchaseValidationReq in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        409: Conflict - ApimodelsErrorResponse (Conflict)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPurchasable.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateInventory)
def admin_update_inventory(
    body: ApimodelsUpdateInventoryReq,
    inventory_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To update inventory (AdminUpdateInventory)


    Updating an inventory.
    Positive value will increase MaxSlots from existing value
    Negative value will decrease MaxSlots from existing value
    Limited slots can not be changed to unlimited, vice versa

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [UPDATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories/{inventoryId}

        method: PUT

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsUpdateInventoryReq in body

        inventory_id: (inventoryId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsInventoryResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateInventory.create(
        body=body,
        inventory_id=inventory_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateInventory)
async def admin_update_inventory_async(
    body: ApimodelsUpdateInventoryReq,
    inventory_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To update inventory (AdminUpdateInventory)


    Updating an inventory.
    Positive value will increase MaxSlots from existing value
    Negative value will decrease MaxSlots from existing value
    Limited slots can not be changed to unlimited, vice versa

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [UPDATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories/{inventoryId}

        method: PUT

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsUpdateInventoryReq in body

        inventory_id: (inventoryId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsInventoryResp (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateInventory.create(
        body=body,
        inventory_id=inventory_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateUserInventoriesByInventoryCode)
def admin_update_user_inventories_by_inventory_code(
    body: ApimodelsUpdateInventoryReq,
    inventory_configuration_code: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To update user inventories by inventory code (AdminUpdateUserInventoriesByInventoryCode)


    Updating user inventories.
    Positive value will increase MaxSlots from existing value
    Negative value will decrease MaxSlots from existing value
    Limited slots can not be changed to unlimited, vice versa

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [UPDATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/users/{userId}/inventoryConfigurations/{inventoryConfigurationCode}/inventories

        method: PUT

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsUpdateInventoryReq in body

        inventory_configuration_code: (inventoryConfigurationCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - List[ApimodelsInventoryResp] (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        401: Unauthorized - ApimodelsErrorResponse (Unauthorized)

        403: Forbidden - ApimodelsErrorResponse (Forbidden)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateUserInventoriesByInventoryCode.create(
        body=body,
        inventory_configuration_code=inventory_configuration_code,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateUserInventoriesByInventoryCode)
async def admin_update_user_inventories_by_inventory_code_async(
    body: ApimodelsUpdateInventoryReq,
    inventory_configuration_code: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To update user inventories by inventory code (AdminUpdateUserInventoriesByInventoryCode)


    Updating user inventories.
    Positive value will increase MaxSlots from existing value
    Negative value will decrease MaxSlots from existing value
    Limited slots can not be changed to unlimited, vice versa

    Permission: ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [UPDATE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/users/{userId}/inventoryConfigurations/{inventoryConfigurationCode}/inventories

        method: PUT

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsUpdateInventoryReq in body

        inventory_configuration_code: (inventoryConfigurationCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - List[ApimodelsInventoryResp] (OK)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        401: Unauthorized - ApimodelsErrorResponse (Unauthorized)

        403: Forbidden - ApimodelsErrorResponse (Forbidden)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateUserInventoriesByInventoryCode.create(
        body=body,
        inventory_configuration_code=inventory_configuration_code,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteInventory)
def delete_inventory(
    body: ApimodelsDeleteInventoryReq,
    inventory_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To delete inventory (deleteInventory)


    Deleting an inventory.
    If an inventory still has items, it cannot be deleted.

    ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [DELETE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories/{inventoryId}

        method: DELETE

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsDeleteInventoryReq in body

        inventory_id: (inventoryId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteInventory.create(
        body=body,
        inventory_id=inventory_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteInventory)
async def delete_inventory_async(
    body: ApimodelsDeleteInventoryReq,
    inventory_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """To delete inventory (deleteInventory)


    Deleting an inventory.
    If an inventory still has items, it cannot be deleted.

    ADMIN:NAMESPACE:{namespace}:USER:{userId}:INVENTORY [DELETE]

    Properties:
        url: /inventory/v1/admin/namespaces/{namespace}/inventories/{inventoryId}

        method: DELETE

        tags: ["Admin Inventories"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelsDeleteInventoryReq in body

        inventory_id: (inventoryId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ApimodelsErrorResponse (Bad Request)

        404: Not Found - ApimodelsErrorResponse (Not Found)

        500: Internal Server Error - ApimodelsErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteInventory.create(
        body=body,
        inventory_id=inventory_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Chat Service."""

__version__ = "0.4.36"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_add_inbox_category import AdminAddInboxCategory
from .admin_delete_inbox_category import AdminDeleteInboxCategory
from .admin_delete_inbox_message import AdminDeleteInboxMessage
from .admin_get_category_schema import AdminGetCategorySchema
from .admin_get_inbox_categories import AdminGetInboxCategories
from .admin_get_inbox_messages import AdminGetInboxMessages
from .admin_get_inbox_messages import (
    ScopeEnum as AdminGetInboxMessagesScopeEnum,
    StatusEnum as AdminGetInboxMessagesStatusEnum,
)
from .admin_get_inbox_stats import AdminGetInboxStats
from .admin_get_inbox_users import AdminGetInboxUsers
from .admin_get_inbox_users import StatusEnum as AdminGetInboxUsersStatusEnum
from .admin_save_inbox_message import AdminSaveInboxMessage
from .admin_send_inbox_message import AdminSendInboxMessage
from .admin_unsend_inbox_message import AdminUnsendInboxMessage
from .admin_update_inbox_category import AdminUpdateInboxCategory
from .admin_update_inbox_message import AdminUpdateInboxMessage



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Match Service V2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.matchmaker_team import MatchmakerTeam
from ..models.models_configuration import ModelsConfiguration
from ..models.models_ds_information import ModelsDSInformation
from ..models.models_user import ModelsUser


class ModelsGameSession(Model):
    """Models game session (models.GameSession)

    Properties:
        attributes: (attributes) REQUIRED Dict[str, Any]

        backfill_ticket_id: (backfillTicketID) REQUIRED str

        configuration: (configuration) REQUIRED ModelsConfiguration

        created_at: (createdAt) REQUIRED str

        ds_information: (DSInformation) REQUIRED ModelsDSInformation

        game_mode: (GameMode) REQUIRED str

        id_: (id) REQUIRED str

        is_full: (IsFull) REQUIRED bool

        join_type: (JoinType) REQUIRED str

        leader_id: (leaderID) REQUIRED str

        match_pool: (matchPool) REQUIRED str

        members: (Members) REQUIRED List[ModelsUser]

        namespace: (namespace) REQUIRED str

        teams: (teams) REQUIRED List[MatchmakerTeam]

        updated_at: (updatedAt) REQUIRED str

        version: (version) REQUIRED int
    """

    # region fields

    attributes: Dict[str, Any]  # REQUIRED
    backfill_ticket_id: str  # REQUIRED
    configuration: ModelsConfiguration  # REQUIRED
    created_at: str  # REQUIRED
    ds_information: ModelsDSInformation  # REQUIRED
    game_mode: str  # REQUIRED
    id_: str  # REQUIRED
    is_full: bool  # REQUIRED
    join_type: str  # REQUIRED
    leader_id: str  # REQUIRED
    match_pool: str  # REQUIRED
    members: List[ModelsUser]  # REQUIRED
    namespace: str  # REQUIRED
    teams: List[MatchmakerTeam]  # REQUIRED
    updated_at: str  # REQUIRED
    version: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_attributes(self, value: Dict[str, Any]) -> ModelsGameSession:
        self.attributes = value
        return self

    def with_backfill_ticket_id(self, value: str) -> ModelsGameSession:
        self.backfill_ticket_id = value
        return self

    def with_configuration(self, value: ModelsConfiguration) -> ModelsGameSession:
        self.configuration = value
        return self

    def with_created_at(self, value: str) -> ModelsGameSession:
        self.created_at = value
        return self

    def with_ds_information(self, value: ModelsDSInformation) -> ModelsGameSession:
        self.ds_information = value
        return self

    def with_game_mode(self, value: str) -> ModelsGameSession:
        self.game_mode = value
        return self

    def with_id(self, value: str) -> ModelsGameSession:
        self.id_ = value
        return self

    def with_is_full(self, value: bool) -> ModelsGameSession:
        self.is_full = value
        return self

    def with_join_type(self, value: str) -> ModelsGameSession:
        self.join_type = value
        return self

    def with_leader_id(self, value: str) -> ModelsGameSession:
        self.leader_id = value
        return self

    def with_match_pool(self, value: str) -> ModelsGameSession:
        self.match_pool = value
        return self

    def with_members(self, value: List[ModelsUser]) -> ModelsGameSession:
        self.members = value
        return self

    def with_namespace(self, value: str) -> ModelsGameSession:
        self.namespace = value
        return self

    def with_teams(self, value: List[MatchmakerTeam]) -> ModelsGameSession:
        self.teams = value
        return self

    def with_updated_at(self, value: str) -> ModelsGameSession:
        self.updated_at = value
        return self

    def with_version(self, value: int) -> ModelsGameSession:
        self.version = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "attributes"):
            result["attributes"] = {str(k0): v0 for k0, v0 in self.attributes.items()}
        elif include_empty:
            result["attributes"] = {}
        if hasattr(self, "backfill_ticket_id"):
            result["backfillTicketID"] = str(self.backfill_ticket_id)
        elif include_empty:
            result["backfillTicketID"] = ""
        if hasattr(self, "configuration"):
            result["configuration"] = self.configuration.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["configuration"] = ModelsConfiguration()
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "ds_information"):
            result["DSInformation"] = self.ds_information.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["DSInformation"] = ModelsDSInformation()
        if hasattr(self, "game_mode"):
            result["GameMode"] = str(self.game_mode)
        elif include_empty:
            result["GameMode"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "is_full"):
            result["IsFull"] = bool(self.is_full)
        elif include_empty:
            result["IsFull"] = False
        if hasattr(self, "join_type"):
            result["JoinType"] = str(self.join_type)
        elif include_empty:
            result["JoinType"] = ""
        if hasattr(self, "leader_id"):
            result["leaderID"] = str(self.leader_id)
        elif include_empty:
            result["leaderID"] = ""
        if hasattr(self, "match_pool"):
            result["matchPool"] = str(self.match_pool)
        elif include_empty:
            result["matchPool"] = ""
        if hasattr(self, "members"):
            result["Members"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.members
            ]
        elif include_empty:
            result["Members"] = []
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "teams"):
            result["teams"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.teams
            ]
        elif include_empty:
            result["teams"] = []
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "version"):
            result["version"] = int(self.version)
        elif include_empty:
            result["version"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        attributes: Dict[str, Any],
        backfill_ticket_id: str,
        configuration: ModelsConfiguration,
        created_at: str,
        ds_information: ModelsDSInformation,
        game_mode: str,
        id_: str,
        is_full: bool,
        join_type: str,
        leader_id: str,
        match_pool: str,
        members: List[ModelsUser],
        namespace: str,
        teams: List[MatchmakerTeam],
        updated_at: str,
        version: int,
        **kwargs,
    ) -> ModelsGameSession:
        instance = cls()
        instance.attributes = attributes
        instance.backfill_ticket_id = backfill_ticket_id
        instance.configuration = configuration
        instance.created_at = created_at
        instance.ds_information = ds_information
        instance.game_mode = game_mode
        instance.id_ = id_
        instance.is_full = is_full
        instance.join_type = join_type
        instance.leader_id = leader_id
        instance.match_pool = match_pool
        instance.members = members
        instance.namespace = namespace
        instance.teams = teams
        instance.updated_at = updated_at
        instance.version = version
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsGameSession:
        instance = cls()
        if not dict_:
            return instance
        if "attributes" in dict_ and dict_["attributes"] is not None:
            instance.attributes = {
                str(k0): v0 for k0, v0 in dict_["attributes"].items()
            }
        elif include_empty:
            instance.attributes = {}
        if "backfillTicketID" in dict_ and dict_["backfillTicketID"] is not None:
            instance.backfill_ticket_id = str(dict_["backfillTicketID"])
        elif include_empty:
            instance.backfill_ticket_id = ""
        if "configuration" in dict_ and dict_["configuration"] is not None:
            instance.configuration = ModelsConfiguration.create_from_dict(
                dict_["configuration"], include_empty=include_empty
            )
        elif include_empty:
            instance.configuration = ModelsConfiguration()
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "DSInformation" in dict_ and dict_["DSInformation"] is not None:
            instance.ds_information = ModelsDSInformation.create_from_dict(
                dict_["DSInformation"], include_empty=include_empty
            )
        elif include_empty:
            instance.ds_information = ModelsDSInformation()
        if "GameMode" in dict_ and dict_["GameMode"] is not None:
            instance.game_mode = str(dict_["GameMode"])
        elif include_empty:
            instance.game_mode = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "IsFull" in dict_ and dict_["IsFull"] is not None:
            instance.is_full = bool(dict_["IsFull"])
        elif include_empty:
            instance.is_full = False
        if "JoinType" in dict_ and dict_["JoinType"] is not None:
            instance.join_type = str(dict_["JoinType"])
        elif include_empty:
            instance.join_type = ""
        if "leaderID" in dict_ and dict_["leaderID"] is not None:
            instance.leader_id = str(dict_["leaderID"])
        elif include_empty:
            instance.leader_id = ""
        if "matchPool" in dict_ and dict_["matchPool"] is not None:
            instance.match_pool = str(dict_["matchPool"])
        elif include_empty:
            instance.match_pool = ""
        if "Members" in dict_ and dict_["Members"] is not None:
            instance.members = [
                ModelsUser.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["Members"]
            ]
        elif include_empty:
            instance.members = []
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "teams" in dict_ and dict_["teams"] is not None:
            instance.teams = [
                MatchmakerTeam.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["teams"]
            ]
        elif include_empty:
            instance.teams = []
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "version" in dict_ and dict_["version"] is not None:
            instance.version = int(dict_["version"])
        elif include_empty:
            instance.version = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsGameSession]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsGameSession]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsGameSession, List[ModelsGameSession], Dict[Any, ModelsGameSession]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "attributes": "attributes",
            "backfillTicketID": "backfill_ticket_id",
            "configuration": "configuration",
            "createdAt": "created_at",
            "DSInformation": "ds_information",
            "GameMode": "game_mode",
            "id": "id_",
            "IsFull": "is_full",
            "JoinType": "join_type",
            "leaderID": "leader_id",
            "matchPool": "match_pool",
            "Members": "members",
            "namespace": "namespace",
            "teams": "teams",
            "updatedAt": "updated_at",
            "version": "version",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "attributes": True,
            "backfillTicketID": True,
            "configuration": True,
            "createdAt": True,
            "DSInformation": True,
            "GameMode": True,
            "id": True,
            "IsFull": True,
            "JoinType": True,
            "leaderID": True,
            "matchPool": True,
            "Members": True,
            "namespace": True,
            "teams": True,
            "updatedAt": True,
            "version": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

# TODO: remove

import logging
from os import PathLike
from typing import Any, Dict, Optional, Tuple, Union

from ._config_repository import ConfigRepository
from ._config_repository import DictConfigRepository
from ._config_repository import DotEnvFileConfigRepository
from ._config_repository import EnvironmentConfigRepository
from ._config_repository import JsonConfigRepository
from ._config_repository import JsonFileConfigRepository
from ._config_repository import MyConfigRepository
from ._config_repository import YamlConfigRepository
from ._config_repository import YamlFileConfigRepository

from ._token_repository import TokenRepository
from ._token_repository import InMemoryTokenRepository
from ._token_repository import MyTokenRepository

from ._http_client import HttpClient
from ._http_client import RequestsHttpClient
from ._http_client import HttpxHttpClient
from ._http_response import HttpResponse
from ._http_utils import get_query_from_http_redirect_response
from ._proto_http_request import ProtoHttpRequest
from ._proto_http_request import create_proto_from_operation

from ._operation import Operation

from ._utils import add_buffered_file_handler_to_logger

_LOGGER = logging.getLogger("accelbyte_py_sdk")

_IS_INITIALIZED: bool = False
_CONFIG_REPOSITORY: Union[None, ConfigRepository] = None
_TOKEN_REPOSITORY: Union[None, TokenRepository] = None
_HTTP_CLIENT: Union[None, HttpClient] = None

_CONFIG_REPOSITORY_IMPL = [
    EnvironmentConfigRepository,
    DictConfigRepository,
    DotEnvFileConfigRepository,
    JsonConfigRepository,
    JsonFileConfigRepository,
    MyConfigRepository,
    YamlConfigRepository,
    YamlFileConfigRepository,
]

_TOKEN_REPOSITORY_IMPL = [
    InMemoryTokenRepository,
    MyTokenRepository,
]

_HTTP_CLIENT_IMPL = [
    RequestsHttpClient,
    HttpxHttpClient,
]


def is_initialized() -> bool:
    return _IS_INITIALIZED


def initialize(options: Union[None, Dict[str, Any]] = None) -> None:
    """Initializes the AccelByte Python SDK.

    Args:
        options (dict): Options used by the SDK. It can contain any of the following:
         -- config (Optional[Union[str, Type[ConfigRepository]]]): Config Repository to use.
         -- config_params (Optional[Tuple[List[Any], Dict[str, Any]]]): Config Repository parameters.
         -- token (Optional[Union[str, Type[TokenRepository]]]): Token Repository to use.
         -- token_params (Optional[Tuple[List[Any], Dict[str, Any]]]): Token Repository parameters.
         -- http (Optional[Union[str, Type[HttpClient]]]): Http Client to use.
         -- http_params (Optional[Tuple[List[Any], Dict[str, Any]]]): Http Client parameters.
         -- log_files (Optional[Dict[str], Tuple[str, PathLike[str], dict]]): Log files.

     Raises:
         ValueError: If 'options.config_params' is not Tuple[List[Any], Dict[str, Any]].
         ValueError: If 'options.config' is not recognized.
         ValueError: If 'options.token_params' is not Tuple[List[Any], Dict[str, Any]].
         ValueError: If 'options.token' is not recognized.
         ValueError: If 'options.http_params' is not Tuple[List[Any], Dict[str, Any]].
         ValueError: If 'options.http' is not recognized.
         ValueError: If 'options.log_files' is not Dict[str], Tuple[str, PathLike[str], dict].
    """

    def is_valid_params(params) -> Tuple[bool, list, dict]:
        if params is None:
            return True, [], {}
        if not isinstance(params, tuple):
            return False, [], {}
        if len(params) != 2:
            return False, [], {}
        args, kwargs = params
        if not isinstance(args, list):
            return False, [], {}
        if not isinstance(kwargs, dict):
            return False, args, {}
        return True, args, kwargs

    global _IS_INITIALIZED
    if _IS_INITIALIZED:
        _LOGGER.warning(f"Already initialized.")
        return

    options = options if options is not None else {}

    # region log files

    if "log_files" in options:
        log_files = options["log_files"]
        if not isinstance(log_files, dict):
            raise ValueError(
                f"Log files must be a Dict[str], Tuple[str, PathLike[str], dict]."
            )
        for additional_scope, log_file_options in log_files.items():
            log_file_kwargs = {}
            if isinstance(log_file_options, dict):
                log_file_kwargs = log_file_options
            elif isinstance(log_file_options, (str, PathLike)):
                log_file_kwargs["filename"] = log_file_options
            if "capacity" not in log_file_kwargs:
                log_file_kwargs["capacity"] = 10
            if additional_scope == "accelbyte_py_sdk":
                additional_scope = None
            elif additional_scope.startswith("accelbyte_py_sdk."):
                additional_scope = additional_scope.removeprefix("accelbyte_py_sdk.")
            log_file_kwargs["additional_scope"] = additional_scope
            add_buffered_file_handler_to_logger(**log_file_kwargs)

    # endregion log files

    # region config repository

    if "config" in options:
        config_repository = options["config"]
        if isinstance(config_repository, str):
            config_params = options.get("config_params")
            is_valid_config_params, config_args, config_kwargs = is_valid_params(
                config_params
            )
            if not is_valid_config_params:
                raise ValueError(
                    f"Config params must be a Tuple[List[Any], Dict[str, Any]]."
                )
            implementation = next(
                (
                    impl
                    for impl in _CONFIG_REPOSITORY_IMPL
                    if impl.__name__ == config_repository
                ),
                None,
            )
            if implementation is None:
                raise ValueError(
                    f"Config repository '{config_repository}' not recognized."
                )
            config_repository = implementation(*config_args, **config_kwargs)
    else:
        config_repository = _CONFIG_REPOSITORY_IMPL[0]()

    set_config_repository(config_repository)

    # endregion config repository

    # region token repository

    if "token" in options:
        token_repository = options["token"]
        if isinstance(token_repository, str):
            token_params = options.get("token_params")
            is_valid_token_params, token_args, token_kwargs = is_valid_params(
                token_params
            )
            if not is_valid_token_params:
                raise ValueError(
                    f"Token params must be a Tuple[List[Any], Dict[str, Any]]."
                )
            implementation = next(
                (
                    impl
                    for impl in _TOKEN_REPOSITORY_IMPL
                    if impl.__name__ == token_repository
                ),
                None,
            )
            if implementation is None:
                raise ValueError(
                    f"Token repository '{token_repository}' not recognized."
                )
            token_repository = implementation(*token_args, **token_kwargs)
    else:
        token_repository = _TOKEN_REPOSITORY_IMPL[0]()

    set_token_repository(token_repository)

    # endregion token repository

    # region http client

    if "http" in options:
        http_client = options["http"]
        if isinstance(http_client, str):
            http_params = options.get("http_params")
            is_valid_http_params, http_args, http_kwargs = is_valid_params(http_params)
            if not is_valid_http_params:
                raise ValueError(
                    f"HTTP params must be a Tuple[List[Any], Dict[str, Any]]."
                )
            implementation = next(
                (impl for impl in _HTTP_CLIENT_IMPL if impl.__name__ == http_client),
                None,
            )
            if implementation is None:
                raise ValueError(f"HTTP Client '{http_client}' not recognized.")
            http_client = implementation(*http_args, **http_kwargs)
    else:
        http_client = _HTTP_CLIENT_IMPL[0]()

    set_http_client(http_client)

    # endregion http client

    _IS_INITIALIZED = True
    _LOGGER.info("AccelByte Python SDK initialized.")


def reset() -> None:
    global _IS_INITIALIZED
    global _CONFIG_REPOSITORY
    global _TOKEN_REPOSITORY
    global _HTTP_CLIENT

    if _HTTP_CLIENT is not None:
        _HTTP_CLIENT.close()

    _IS_INITIALIZED = False
    _CONFIG_REPOSITORY = None
    _TOKEN_REPOSITORY = None
    _HTTP_CLIENT = None


# region ConfigRepository


def get_config_repository() -> ConfigRepository:
    if _CONFIG_REPOSITORY is None:
        raise ValueError("Config repository not set.")
    return _CONFIG_REPOSITORY


def set_config_repository(config_repository: ConfigRepository) -> None:
    if not isinstance(config_repository, ConfigRepository):
        raise TypeError(
            f"Config repository '{type(config_repository).__name__}' not valid."
        )
    global _CONFIG_REPOSITORY
    _CONFIG_REPOSITORY = config_repository


def get_base_url() -> Tuple[Union[None, str], Union[None, HttpResponse]]:
    config_repo = _CONFIG_REPOSITORY
    if not config_repo:
        return None, HttpResponse.create_config_repo_not_found_error()
    base_url = config_repo.get_base_url()
    if not base_url:
        return None, HttpResponse.create_base_url_not_set_error()
    return base_url, None


def get_client_auth() -> Tuple[Union[None, Tuple[str, str]], Union[None, HttpResponse]]:
    config_repo = _CONFIG_REPOSITORY
    if not config_repo:
        return None, HttpResponse.create_config_repo_not_found_error()
    client_id = config_repo.get_client_id()
    client_secret = config_repo.get_client_secret()
    if not client_id:
        return None, HttpResponse.create_client_not_registered_error()
    client_secret = client_secret or ""
    return (client_id, client_secret), None


def get_client_id() -> Tuple[Union[None, str], Union[None, HttpResponse]]:
    config_repo = _CONFIG_REPOSITORY
    if not config_repo:
        return None, HttpResponse.create_config_repo_not_found_error()
    client_id = config_repo.get_client_id()
    if not client_id:
        return None, HttpResponse.create_client_not_registered_error()
    return client_id, None


def get_client_secret() -> Tuple[Union[None, str], Union[None, HttpResponse]]:
    config_repo = _CONFIG_REPOSITORY
    if not config_repo:
        return None, HttpResponse.create_config_repo_not_found_error()
    client_secret = config_repo.get_client_secret() or ""
    return client_secret, None


def get_namespace() -> Tuple[Union[None, str], Union[None, HttpResponse]]:
    config_repo = _CONFIG_REPOSITORY
    if not config_repo:
        return None, HttpResponse.create_config_repo_not_found_error()
    namespace = config_repo.get_namespace()
    if not namespace:
        return None, HttpResponse.create_error(400, "Namespace not found.")
    return namespace, None


def get_app_name() -> Tuple[Union[None, str], Union[None, HttpResponse]]:
    config_repo = _CONFIG_REPOSITORY
    if not config_repo:
        return None, HttpResponse.create_config_repo_not_found_error()
    app_name = config_repo.get_app_name()
    if not app_name:
        return None, HttpResponse.create_error(400, "App name not found.")
    return app_name, None


def get_app_version() -> Tuple[Union[None, str], Union[None, HttpResponse]]:
    config_repo = _CONFIG_REPOSITORY
    if not config_repo:
        return None, HttpResponse.create_config_repo_not_found_error()
    app_version = config_repo.get_app_version()
    if not app_version:
        return None, HttpResponse.create_error(400, "App version not found.")
    return app_version, None


# endregion ConfigRepository


# region TokenRepository


def get_token_repository() -> TokenRepository:
    if _TOKEN_REPOSITORY is None:
        raise ValueError("Token repository not set.")
    return _TOKEN_REPOSITORY


def set_token_repository(token_repository: TokenRepository) -> None:
    if not isinstance(token_repository, TokenRepository):
        raise TypeError(
            f"Token repository '{type(token_repository).__name__}' not valid."
        )
    global _TOKEN_REPOSITORY
    _TOKEN_REPOSITORY = token_repository


def get_access_token() -> Tuple[Union[None, str], Union[None, HttpResponse]]:
    token_repo = _TOKEN_REPOSITORY
    if not token_repo:
        return None, HttpResponse.create_token_repo_not_found_error()
    token = token_repo.get_token()
    if not token:
        return None, HttpResponse.create_token_not_found_error()
    if hasattr(token, "access_token"):
        return str(token.access_token), None
    elif hasattr(token, "__iter__") and "access_token" in token:
        return str(token["access_token"]), None
    else:
        return None, HttpResponse.create_token_not_found_error()


def remove_token() -> Tuple[None, Union[None, HttpResponse]]:
    token_repo = _TOKEN_REPOSITORY
    if not token_repo:
        return None, HttpResponse.create_token_repo_not_found_error()
    success = token_repo.remove_token()
    if not success:
        return None, HttpResponse.create_error(400, "Failed to remove token.")
    return None, None


def set_token(token: Any) -> Tuple[None, Union[None, HttpResponse]]:
    token_repo = _TOKEN_REPOSITORY
    if not token_repo:
        return None, HttpResponse.create_token_repo_not_found_error()
    success = token_repo.store_token(token)
    if not success:
        return None, HttpResponse.create_error(400, "Failed to set token.")
    return None, None


def _is_valid_token(token: Any) -> bool:
    if token is None:
        return False
    access_token_key = "access_token"
    if hasattr(token, access_token_key):  # in attr
        return True
    if isinstance(token, dict) and access_token_key in token:  # in dict
        return True
    return False


def _try_set_token(token: Any) -> Tuple[bool, Union[None, HttpResponse]]:
    if token is None:
        return False, HttpResponse.create_error(400, "Empty token.")
    if not _is_valid_token(token):
        return False, HttpResponse.create_error(
            400, "Failed to set token. The token is not valid."
        )
    _, error = set_token(token)
    if error:
        return True, error
    return True, None


# endregion TokenRepository


# region HttpClient


def get_http_client() -> HttpClient:
    if _HTTP_CLIENT is None:
        raise ValueError("HTTP client not set.")
    return _HTTP_CLIENT


def set_http_client(http_client: HttpClient) -> None:
    if not isinstance(http_client, HttpClient):
        raise TypeError(f"HTTP Client '{type(http_client).__name__}' not valid.")
    global _HTTP_CLIENT
    _HTTP_CLIENT = http_client


def _pre_run_request(
    operation: Operation,
    base_url: Optional[str] = "",
    additional_headers: Optional[Dict[str, str]] = None,
    additional_headers_override: bool = True,
    config_repo: Optional[ConfigRepository] = None,
    token_repo: Optional[TokenRepository] = None,
    **kwargs,
) -> Tuple[Any, Any]:
    if not config_repo:
        if not _CONFIG_REPOSITORY:
            return None, HttpResponse.create_config_repo_not_found_error()
        config_repo = _CONFIG_REPOSITORY
    if not token_repo:
        if not _TOKEN_REPOSITORY:
            return None, HttpResponse.create_token_repo_not_found_error()
        token_repo = _TOKEN_REPOSITORY
    if not base_url:
        base_url = config_repo.get_base_url()

    proto, error = create_proto_from_operation(
        operation=operation,
        base_url=base_url,
        additional_headers=additional_headers,
        additional_headers_override=additional_headers_override,
        config_repo=config_repo,
        token_repo=token_repo,
        **kwargs,
    )
    return proto, error


def _post_run_request(
    operation: Operation, response: Any, error: Any
) -> Tuple[Any, Any]:
    success, error = operation.parse_response(*response)
    if error:
        return None, error

    if operation.has_redirects() and operation.location_query:
        query, error = get_query_from_http_redirect_response(
            success, operation.location_query
        )
        if error:
            return None, error
        else:
            return query, None

    # TODO(elmer): still not a fan of this bit
    is_valid_token, error = _try_set_token(success)
    if error and is_valid_token:
        return None, error

    return success, None


def run_proto_request(
    proto: ProtoHttpRequest,
    has_redirects: bool = False,
    http_client: Optional[HttpClient] = None,
    **kwargs,
) -> Tuple[Any, Any]:
    if has_redirects and "allow_redirects" not in kwargs:
        kwargs["allow_redirects"] = False
    if not http_client:
        if not _HTTP_CLIENT:
            return None, HttpResponse.create_http_client_not_found_error()
        http_client = _HTTP_CLIENT

    request = http_client.create_request(proto=proto)

    raw_response, error = http_client.send_request(request, **kwargs)
    if error:
        return None, error

    response, error = http_client.handle_response(raw_response, **kwargs)
    return response, error


async def run_proto_request_async(
    proto: ProtoHttpRequest,
    has_redirects: bool = False,
    http_client: Optional[HttpClient] = None,
    **kwargs,
) -> Tuple[Any, Any]:
    if has_redirects and "allow_redirects" not in kwargs:
        kwargs["allow_redirects"] = False
    if not http_client:
        if not _HTTP_CLIENT:
            return None, HttpResponse.create_http_client_not_found_error()
        http_client = _HTTP_CLIENT

    request = http_client.create_request(proto=proto)

    raw_response, error = await http_client.send_request_async(request, **kwargs)
    if error:
        return None, error

    response, error = http_client.handle_response(raw_response, **kwargs)
    return response, error


def run_request(
    operation: Operation,
    base_url: Optional[str] = "",
    additional_headers: Optional[Dict[str, str]] = None,
    additional_headers_override: bool = True,
    config_repo: Optional[ConfigRepository] = None,
    token_repo: Optional[TokenRepository] = None,
    http_client: Optional[HttpClient] = None,
    **kwargs,
) -> Tuple[Any, Any]:
    proto, error = _pre_run_request(
        operation=operation,
        base_url=base_url,
        additional_headers=additional_headers,
        additional_headers_override=additional_headers_override,
        config_repo=config_repo,
        token_repo=token_repo,
        http_client=http_client,
        **kwargs,
    )
    if error:
        return None, error

    response, error = run_proto_request(
        proto=proto,
        has_redirects=operation.has_redirects(),
        http_client=http_client,
    )
    if error:
        return None, error

    result, error = _post_run_request(
        operation=operation, response=response, error=error
    )
    return result, error


async def run_request_async(
    operation: Operation,
    base_url: Optional[str] = "",
    additional_headers: Optional[Dict[str, str]] = None,
    additional_headers_override: bool = True,
    config_repo: Optional[ConfigRepository] = None,
    token_repo: Optional[TokenRepository] = None,
    http_client: Optional[HttpClient] = None,
    **kwargs,
) -> Tuple[Any, Any]:
    proto, error = _pre_run_request(
        operation=operation,
        base_url=base_url,
        additional_headers=additional_headers,
        additional_headers_override=additional_headers_override,
        config_repo=config_repo,
        token_repo=token_repo,
        http_client=http_client,
        **kwargs,
    )
    if error:
        return None, error

    response, error = await run_proto_request_async(
        proto=proto,
        has_redirects=operation.has_redirects(),
        http_client=http_client,
    )
    if error:
        return None, error

    result, error = _post_run_request(
        operation=operation, response=response, error=error
    )
    return result, error


# endregion HttpClient



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class StatusEnum(StrEnum):
    ACTIVE = "ACTIVE"
    BILLING_RETRY_PERIOD = "BILLING_RETRY_PERIOD"
    CANCELED = "CANCELED"
    EXPIRED = "EXPIRED"
    IN_GRACE_PERIOD = "IN_GRACE_PERIOD"
    ON_HOLD = "ON_HOLD"
    PAUSED = "PAUSED"
    PENDING = "PENDING"
    REVOKED = "REVOKED"
    UNKNOWN = "UNKNOWN"


class ThirdPartySubscriptionTransactionHistoryInfo(Model):
    """Third party subscription transaction history info (ThirdPartySubscriptionTransactionHistoryInfo)

    Properties:
        id_: (id) REQUIRED str

        namespace: (namespace) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        third_party_subscription_id: (thirdPartySubscriptionId) REQUIRED str

        user_id: (userId) REQUIRED str

        created_at: (createdAt) OPTIONAL str

        expired_at: (expiredAt) OPTIONAL str

        last_transaction_id: (lastTransactionId) OPTIONAL str

        operator: (operator) OPTIONAL str

        reason: (reason) OPTIONAL str

        subscription_product_id: (subscriptionProductId) OPTIONAL str

        updated_at: (updatedAt) OPTIONAL str
    """

    # region fields

    id_: str  # REQUIRED
    namespace: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    third_party_subscription_id: str  # REQUIRED
    user_id: str  # REQUIRED
    created_at: str  # OPTIONAL
    expired_at: str  # OPTIONAL
    last_transaction_id: str  # OPTIONAL
    operator: str  # OPTIONAL
    reason: str  # OPTIONAL
    subscription_product_id: str  # OPTIONAL
    updated_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_id(self, value: str) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.id_ = value
        return self

    def with_namespace(
        self, value: str
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.namespace = value
        return self

    def with_status(
        self, value: Union[str, StatusEnum]
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.status = value
        return self

    def with_third_party_subscription_id(
        self, value: str
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.third_party_subscription_id = value
        return self

    def with_user_id(self, value: str) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.user_id = value
        return self

    def with_created_at(
        self, value: str
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.created_at = value
        return self

    def with_expired_at(
        self, value: str
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.expired_at = value
        return self

    def with_last_transaction_id(
        self, value: str
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.last_transaction_id = value
        return self

    def with_operator(self, value: str) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.operator = value
        return self

    def with_reason(self, value: str) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.reason = value
        return self

    def with_subscription_product_id(
        self, value: str
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.subscription_product_id = value
        return self

    def with_updated_at(
        self, value: str
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "third_party_subscription_id"):
            result["thirdPartySubscriptionId"] = str(self.third_party_subscription_id)
        elif include_empty:
            result["thirdPartySubscriptionId"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "expired_at"):
            result["expiredAt"] = str(self.expired_at)
        elif include_empty:
            result["expiredAt"] = ""
        if hasattr(self, "last_transaction_id"):
            result["lastTransactionId"] = str(self.last_transaction_id)
        elif include_empty:
            result["lastTransactionId"] = ""
        if hasattr(self, "operator"):
            result["operator"] = str(self.operator)
        elif include_empty:
            result["operator"] = ""
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "subscription_product_id"):
            result["subscriptionProductId"] = str(self.subscription_product_id)
        elif include_empty:
            result["subscriptionProductId"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        id_: str,
        namespace: str,
        status: Union[str, StatusEnum],
        third_party_subscription_id: str,
        user_id: str,
        created_at: Optional[str] = None,
        expired_at: Optional[str] = None,
        last_transaction_id: Optional[str] = None,
        operator: Optional[str] = None,
        reason: Optional[str] = None,
        subscription_product_id: Optional[str] = None,
        updated_at: Optional[str] = None,
        **kwargs,
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        instance = cls()
        instance.id_ = id_
        instance.namespace = namespace
        instance.status = status
        instance.third_party_subscription_id = third_party_subscription_id
        instance.user_id = user_id
        if created_at is not None:
            instance.created_at = created_at
        if expired_at is not None:
            instance.expired_at = expired_at
        if last_transaction_id is not None:
            instance.last_transaction_id = last_transaction_id
        if operator is not None:
            instance.operator = operator
        if reason is not None:
            instance.reason = reason
        if subscription_product_id is not None:
            instance.subscription_product_id = subscription_product_id
        if updated_at is not None:
            instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ThirdPartySubscriptionTransactionHistoryInfo:
        instance = cls()
        if not dict_:
            return instance
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if (
            "thirdPartySubscriptionId" in dict_
            and dict_["thirdPartySubscriptionId"] is not None
        ):
            instance.third_party_subscription_id = str(
                dict_["thirdPartySubscriptionId"]
            )
        elif include_empty:
            instance.third_party_subscription_id = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "expiredAt" in dict_ and dict_["expiredAt"] is not None:
            instance.expired_at = str(dict_["expiredAt"])
        elif include_empty:
            instance.expired_at = ""
        if "lastTransactionId" in dict_ and dict_["lastTransactionId"] is not None:
            instance.last_transaction_id = str(dict_["lastTransactionId"])
        elif include_empty:
            instance.last_transaction_id = ""
        if "operator" in dict_ and dict_["operator"] is not None:
            instance.operator = str(dict_["operator"])
        elif include_empty:
            instance.operator = ""
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if (
            "subscriptionProductId" in dict_
            and dict_["subscriptionProductId"] is not None
        ):
            instance.subscription_product_id = str(dict_["subscriptionProductId"])
        elif include_empty:
            instance.subscription_product_id = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ThirdPartySubscriptionTransactionHistoryInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ThirdPartySubscriptionTransactionHistoryInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ThirdPartySubscriptionTransactionHistoryInfo,
        List[ThirdPartySubscriptionTransactionHistoryInfo],
        Dict[Any, ThirdPartySubscriptionTransactionHistoryInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "id": "id_",
            "namespace": "namespace",
            "status": "status",
            "thirdPartySubscriptionId": "third_party_subscription_id",
            "userId": "user_id",
            "createdAt": "created_at",
            "expiredAt": "expired_at",
            "lastTransactionId": "last_transaction_id",
            "operator": "operator",
            "reason": "reason",
            "subscriptionProductId": "subscription_product_id",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "id": True,
            "namespace": True,
            "status": True,
            "thirdPartySubscriptionId": True,
            "userId": True,
            "createdAt": False,
            "expiredAt": False,
            "lastTransactionId": False,
            "operator": False,
            "reason": False,
            "subscriptionProductId": False,
            "updatedAt": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": [
                "ACTIVE",
                "BILLING_RETRY_PERIOD",
                "CANCELED",
                "EXPIRED",
                "IN_GRACE_PERIOD",
                "ON_HOLD",
                "PAUSED",
                "PENDING",
                "REVOKED",
                "UNKNOWN",
            ],
        }

    # endregion static methods
